@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2018 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -48,7 +48,11 @@ import javax.security.auth.callback.*;
 
 import com.sun.mail.iap.*;
 import com.sun.mail.imap.*;
-import com.sun.mail.util.*;
+import com.sun.mail.util.PropUtil;
+import com.sun.mail.util.MailLogger;
+import com.sun.mail.util.ASCIIUtility;
+import com.sun.mail.util.BASE64EncoderStream;
+import com.sun.mail.util.BASE64DecoderStream;
 
 /**
  * This class contains a single method that does authentication using
@@ -85,12 +89,13 @@ public class IMAPSaslAuthenticator implements SaslAuthenticator {
 	this.host = host;
     }
 
+    @Override
     public boolean authenticate(String[] mechs, final String realm,
 				final String authzid, final String u,
 				final String p) throws ProtocolException {
 
 	synchronized (pr) {	// authenticate method should be synchronized
-	List<Response> v = new ArrayList<Response>();
+	List<Response> v = new ArrayList<>();
 	String tag = null;
 	Response r = null;
 	boolean done = false;
@@ -103,6 +108,7 @@ public class IMAPSaslAuthenticator implements SaslAuthenticator {
 
 	SaslClient sc;
 	CallbackHandler cbh = new CallbackHandler() {
+		@Override
 	    public void handle(Callback[] callbacks) {
 		if (logger.isLoggable(Level.FINE))
 		    logger.fine("SASL callback length: " + callbacks.length);
@@ -259,13 +265,18 @@ public class IMAPSaslAuthenticator implements SaslAuthenticator {
 	    }
 	}
 
-	/* Dispatch untagged responses.
+	Response[] responses = v.toArray(new Response[v.size()]);
+
+	// handle an illegal but not uncommon untagged CAPABILTY response
+	pr.handleCapabilityResponse(responses);
+
+	/*
+	 * Dispatch untagged responses.
 	 * NOTE: in our current upper level IMAP classes, we add the
 	 * responseHandler to the Protocol object only *after* the 
 	 * connection has been authenticated. So, for now, the below
 	 * code really ends up being just a no-op.
 	 */
-	Response[] responses = v.toArray(new Response[v.size()]);
 	pr.notifyResponseHandlers(responses);
 
 	// Handle the final OK, NO, BAD or BYE response
