@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -74,7 +74,7 @@ public class MessageSet {
      * @return		array of MessageSet objects
      */
     public static MessageSet[] createMessageSets(int[] msgs) {
-	List<MessageSet> v = new ArrayList<MessageSet>();
+	List<MessageSet> v = new ArrayList<>();
 	int i,j;
 
 	for (i=0; i < msgs.length; i++) {
@@ -104,7 +104,7 @@ public class MessageSet {
 	    return null; 
 
 	int i = 0;  // msgset index
-	StringBuffer s = new StringBuffer();
+	StringBuilder s = new StringBuilder();
 	int size = msgsets.length;
 	int start, end;
 
