@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2018 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -47,7 +47,13 @@ import java.security.*;
 import java.util.logging.Level;
 import javax.net.ssl.SSLSocket;
 
-import com.sun.mail.util.*;
+import com.sun.mail.util.PropUtil;
+import com.sun.mail.util.MailLogger;
+import com.sun.mail.util.SocketFetcher;
+import com.sun.mail.util.LineInputStream;
+import com.sun.mail.util.TraceInputStream;
+import com.sun.mail.util.TraceOutputStream;
+import com.sun.mail.util.SharedByteArrayOutputStream;
 
 class Response {
     boolean ok = false;		// true if "+OK"
@@ -113,19 +119,11 @@ class Protocol {
 	    initStreams();
 	    r = simpleCommand(null);
 	} catch (IOException ioe) {
-	    try {
-		socket.close();
-	    } finally {
-		throw ioe;
-	    }
+	    throw cleanupAndThrow(socket, ioe);
 	}
 
 	if (!r.ok) {
-	    try {
-		socket.close();
-	    } finally {
-		throw new IOException("Connect failed");
-	    }
+	    throw cleanupAndThrow(socket, new IOException("Connect failed"));
 	}
 	if (enableAPOP && r.data != null) {
 	    int challStart = r.data.indexOf('<');	// start of challenge
@@ -145,6 +143,30 @@ class Protocol {
 	    logger.config("PIPELINING enabled");
     }
 
+    private static IOException cleanupAndThrow(Socket socket, IOException ife) {
+	try {
+	    socket.close();
+	} catch (Throwable thr) {
+	    if (isRecoverable(thr)) {
+		ife.addSuppressed(thr);
+	    } else {
+		thr.addSuppressed(ife);
+		if (thr instanceof Error) {
+		    throw (Error) thr;
+		}
+		if (thr instanceof RuntimeException) {
+		    throw (RuntimeException) thr;
+		}
+		throw new RuntimeException("unexpected exception", thr);
+	    }
+	}
+	return ife;
+    }
+
+    private static boolean isRecoverable(Throwable t) {
+	return (t instanceof Exception) || (t instanceof LinkageError);
+    }
+
     /**
      * Get the value of a boolean property.
      * Print out the value if logging is enabled.
@@ -176,6 +198,7 @@ class Protocol {
 			new OutputStreamWriter(traceOutput, "iso-8859-1")));
     }
 
+    @Override
     protected void finalize() throws Throwable {
 	try {
 	    if (socket != null)	// Forgot to logout ?!
@@ -194,7 +217,7 @@ class Protocol {
 	    return;
 	}
 
-	capabilities = new HashMap<String, String>(10);
+	capabilities = new HashMap<>(10);
 	BufferedReader r = null;
 	try {
 	    r = new BufferedReader(new InputStreamReader(in, "us-ascii"));
