@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -44,6 +44,7 @@ import java.security.*;
 import java.net.*;
 import java.io.*;
 import java.nio.channels.SocketChannel;
+import java.nio.charset.StandardCharsets;
 import java.lang.reflect.*;
 import java.util.*;
 import java.util.regex.*;
@@ -78,7 +79,7 @@ public class SocketFetcher {
     /**
      * This method returns a Socket.  Properties control the use of
      * socket factories and other socket characteristics.  The properties
-     * used are: <p>
+     * used are:
      * <ul>
      * <li> <i>prefix</i>.socketFactory
      * <li> <i>prefix</i>.socketFactory.class
@@ -139,6 +140,8 @@ public class SocketFetcher {
      * @param props Properties object containing socket properties
      * @param prefix Property name prefix, e.g., "mail.imap"
      * @param useSSL use the SSL socket factory as the default
+     * @return		the Socket
+     * @exception	IOException	for I/O errors
      */
     public static Socket getSocket(String host, int port, Properties props,
 				String prefix, boolean useSSL)
@@ -273,10 +276,29 @@ public class SocketFetcher {
 		", connection timeout " + cto + ", timeout " + to +
 		", socket factory " + sf + ", useSSL " + useSSL);
 		
-	String socksHost = props.getProperty(prefix + ".socks.host", null);
+	String proxyHost = props.getProperty(prefix + ".proxy.host", null);
+	int proxyPort = 80;
+	String socksHost = null;
 	int socksPort = 1080;
 	String err = null;
-	if (socksHost != null) {
+
+	if (proxyHost != null) {
+	    int i = proxyHost.indexOf(':');
+	    if (i >= 0) {
+		proxyHost = proxyHost.substring(0, i);
+		try {
+		    proxyPort = Integer.parseInt(proxyHost.substring(i + 1));
+		} catch (NumberFormatException ex) {
+		    // ignore it
+		}
+	    }
+	    proxyPort = PropUtil.getIntProperty(props,
+					prefix + ".proxy.port", proxyPort);
+	    err = "Using web proxy host, port: " + proxyHost + ", " + proxyPort;
+	    if (logger.isLoggable(Level.FINER))
+		logger.finer("web proxy host " + proxyHost + ", port " + proxyPort);
+	} else if ((socksHost =
+		    props.getProperty(prefix + ".socks.host", null)) != null) {
 	    int i = socksHost.indexOf(':');
 	    if (i >= 0) {
 		socksHost = socksHost.substring(0, i);
@@ -323,7 +345,9 @@ public class SocketFetcher {
 	    socket.bind(new InetSocketAddress(localaddr, localport));
 	try {
 	    logger.finest("connecting...");
-	    if (cto >= 0)
+	    if (proxyHost != null)
+		proxyConnect(socket, proxyHost, proxyPort, host, port, cto);
+	    else if (cto >= 0)
 		socket.connect(new InetSocketAddress(host, port), cto);
 	    else
 		socket.connect(new InetSocketAddress(host, port));
@@ -337,7 +361,8 @@ public class SocketFetcher {
 	 * If we want an SSL connection and we didn't get an SSLSocket,
 	 * wrap our plain Socket with an SSLSocket.
 	 */
-	if (useSSL && !(socket instanceof SSLSocket)) {
+	if ((useSSL || sf instanceof SSLSocketFactory) &&
+		!(socket instanceof SSLSocket)) {
 	    String trusted;
 	    SSLSocketFactory ssf;
 	    if ((trusted = props.getProperty(prefix + ".ssl.trust")) != null) {
@@ -385,7 +410,7 @@ public class SocketFetcher {
 	// dynamically load the class 
 
 	ClassLoader cl = getContextClassLoader();
-	Class clsSockFact = null;
+	Class<?> clsSockFact = null;
 	if (cl != null) {
 	    try {
 		clsSockFact = Class.forName(sfClass, false, cl);
@@ -395,7 +420,7 @@ public class SocketFetcher {
 	    clsSockFact = Class.forName(sfClass);
 	// get & invoke the getDefault() method
 	Method mthGetDefault = clsSockFact.getMethod("getDefault", 
-						     new Class[]{});
+						     new Class<?>[]{});
 	SocketFactory sf = (SocketFactory)
 	    mthGetDefault.invoke(new Object(), new Object[]{});
 	return sf;
@@ -406,7 +431,13 @@ public class SocketFetcher {
      * Supports the "STARTTLS" command in many protocols.
      * This version for compatibility with possible third party code
      * that might've used this API even though it shouldn't.
+     *
+     * @param	socket	the existing socket
+     * @return		the wrapped Socket
+     * @exception	IOException	for I/O errors
+     * @deprecated
      */
+    @Deprecated
     public static Socket startTLS(Socket socket) throws IOException {
 	return startTLS(socket, new Properties(), "socket");
     }
@@ -416,7 +447,15 @@ public class SocketFetcher {
      * Supports the "STARTTLS" command in many protocols.
      * This version for compatibility with possible third party code
      * that might've used this API even though it shouldn't.
+     *
+     * @param	socket	the existing socket
+     * @param	props	the properties
+     * @param	prefix	the property prefix
+     * @return		the wrapped Socket
+     * @exception	IOException	for I/O errors
+     * @deprecated
      */
+    @Deprecated
     public static Socket startTLS(Socket socket, Properties props,
 				String prefix) throws IOException {
 	InetAddress a = socket.getInetAddress();
@@ -427,6 +466,13 @@ public class SocketFetcher {
     /**
      * Start TLS on an existing socket.
      * Supports the "STARTTLS" command in many protocols.
+     *
+     * @param	socket	the existing socket
+     * @param	host	the host the socket is connected to
+     * @param	props	the properties
+     * @param	prefix	the property prefix
+     * @return		the wrapped Socket
+     * @exception	IOException	for I/O errors
      */
     public static Socket startTLS(Socket socket, String host, Properties props,
 				String prefix) throws IOException {
@@ -548,7 +594,7 @@ public class SocketFetcher {
 	    if (logger.isLoggable(Level.FINER))
 		logger.finer("SSL enabled protocols before " +
 		    Arrays.asList(prots));
-	    List<String> eprots = new ArrayList<String>();
+	    List<String> eprots = new ArrayList<>();
 	    for (int i = 0; i < prots.length; i++) {
 		if (prots[i] != null && !prots[i].startsWith("SSL"))
 		    eprots.add(prots[i]);
@@ -583,15 +629,36 @@ public class SocketFetcher {
 	if (sf instanceof MailSSLSocketFactory) {
 	    MailSSLSocketFactory msf = (MailSSLSocketFactory)sf;
 	    if (!msf.isServerTrusted(host, sslsocket)) {
-		try {
-		    sslsocket.close();
-		} finally {
-		    throw new IOException("Server is not trusted: " + host);
-		}
+		throw cleanupAndThrow(sslsocket,
+			new IOException("Server is not trusted: " + host));
 	    }
 	}
     }
 
+    private static IOException cleanupAndThrow(Socket socket, IOException ife) {
+	try {
+	    socket.close();
+	} catch (Throwable thr) {
+	    if (isRecoverable(thr)) {
+		ife.addSuppressed(thr);
+	    } else {
+		thr.addSuppressed(ife);
+		if (thr instanceof Error) {
+		    throw (Error) thr;
+		}
+		if (thr instanceof RuntimeException) {
+		    throw (RuntimeException) thr;
+		}
+		throw new RuntimeException("unexpected exception", thr);
+	    }
+	}
+	return ife;
+    }
+
+    private static boolean isRecoverable(Throwable t) {
+	return (t instanceof Exception) || (t instanceof LinkageError);
+    }
+
     /**
      * Check the server from the Socket connection against the server name(s)
      * as expressed in the server certificate (RFC 2595 check).
@@ -643,12 +710,12 @@ public class SocketFetcher {
 	 * in the JDK we're running on.
 	 */
 	try {
-	    Class hnc = Class.forName("sun.security.util.HostnameChecker");
+	    Class<?> hnc = Class.forName("sun.security.util.HostnameChecker");
 	    // invoke HostnameChecker.getInstance(HostnameChecker.TYPE_LDAP)
 	    // HostnameChecker.TYPE_LDAP == 2
 	    // LDAP requires the same regex handling as we need
 	    Method getInstance = hnc.getMethod("getInstance", 
-					new Class[] { byte.class });
+					new Class<?>[] { byte.class });
 	    Object hostnameChecker = getInstance.invoke(new Object(),
 					new Object[] { Byte.valueOf((byte)2) });
 
@@ -656,7 +723,7 @@ public class SocketFetcher {
 	    if (logger.isLoggable(Level.FINER))
 		logger.finer("using sun.security.util.HostnameChecker");
 	    Method match = hnc.getMethod("match",
-			new Class[] { String.class, X509Certificate.class });
+			new Class<?>[] { String.class, X509Certificate.class });
 	    try {
 		match.invoke(hostnameChecker, new Object[] { server, cert });
 		return true;
@@ -679,11 +746,11 @@ public class SocketFetcher {
 	     * XXX - only checks DNS names, should also handle
 	     * case where server name is a literal IP address
 	     */
-	    Collection names = cert.getSubjectAlternativeNames();
+	    Collection<List<?>> names = cert.getSubjectAlternativeNames();
 	    if (names != null) {
 		boolean foundName = false;
-		for (Iterator it = names.iterator(); it.hasNext(); ) {
-		    List nameEnt = (List)it.next();
+		for (Iterator<List<?>> it = names.iterator(); it.hasNext(); ) {
+		    List<?> nameEnt = it.next();
 		    Integer type = (Integer)nameEnt.get(0);
 		    if (type.intValue() == 2) {	// 2 == dNSName
 			foundName = true;
@@ -736,16 +803,78 @@ public class SocketFetcher {
 	    return server.equalsIgnoreCase(name);
     }
 
+    /**
+     * Use the HTTP CONNECT protocol to connect to a
+     * site through an HTTP proxy server. <p>
+     *
+     * Protocol is roughly:
+     * <pre>
+     * CONNECT <host>:<port> HTTP/1.1
+     * Host: <host>:<port>
+     * <blank line>
+     *
+     * HTTP/1.1 200 Connect established
+     * <headers>
+     * <blank line>
+     * </pre>
+     */
+    private static void proxyConnect(Socket socket,
+				String proxyHost, int proxyPort,
+				String host, int port, int cto)
+				throws IOException {
+	if (logger.isLoggable(Level.FINE))
+	    logger.fine("connecting through proxy " +
+			proxyHost + ":" + proxyPort + " to " +
+			host + ":" + port);
+	if (cto >= 0)
+	    socket.connect(new InetSocketAddress(proxyHost, proxyPort), cto);
+	else
+	    socket.connect(new InetSocketAddress(proxyHost, proxyPort));
+	PrintStream os = new PrintStream(socket.getOutputStream(), false,
+					    StandardCharsets.UTF_8.name());
+	os.print("CONNECT " + host + ":" + port + " HTTP/1.1\r\n");
+	os.print("Host: " + host + ":" + port + "\r\n\r\n");
+	os.flush();
+	BufferedReader r = new BufferedReader(new InputStreamReader(
+			    socket.getInputStream(), StandardCharsets.UTF_8));
+	String line;
+	boolean first = true;
+	while ((line = r.readLine()) != null) {
+	    if (line.length() == 0)
+		break;
+	    logger.finest(line);
+	    if (first) {
+		StringTokenizer st = new StringTokenizer(line);
+		String http = st.nextToken();
+		String code = st.nextToken();
+		if (!code.equals("200")) {
+		    try {
+			socket.close();
+		    } catch (IOException ioex) {
+			// ignored
+		    }
+		    ConnectException ex = new ConnectException(
+			"connection through proxy " +
+			proxyHost + ":" + proxyPort + " to " +
+			host + ":" + port + " failed: " + line);
+		    logger.log(Level.FINE, "connect failed", ex);
+		    throw ex;
+		}
+		first = false;
+	    }
+	}
+    }
+
     /**
      * Parse a string into whitespace separated tokens
      * and return the tokens in an array.
      */
     private static String[] stringArray(String s) {
 	StringTokenizer st = new StringTokenizer(s);
-	List tokens = new ArrayList();
+	List<String> tokens = new ArrayList<>();
 	while (st.hasMoreTokens())
 	    tokens.add(st.nextToken());
-	return (String[])tokens.toArray(new String[tokens.size()]);
+	return tokens.toArray(new String[tokens.size()]);
     }
 
     /**
@@ -754,9 +883,10 @@ public class SocketFetcher {
      * Thread.getContextClassLoader method.
      */
     private static ClassLoader getContextClassLoader() {
-	return (ClassLoader)
-		AccessController.doPrivileged(new PrivilegedAction() {
-	    public Object run() {
+	return
+	AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
+	    @Override
+	    public ClassLoader run() {
 		ClassLoader cl = null;
 		try {
 		    cl = Thread.currentThread().getContextClassLoader();
