@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -116,7 +116,7 @@ import javax.mail.event.*;
  * @author Bill Shannon
  */
 
-public abstract class Folder {
+public abstract class Folder implements AutoCloseable {
 
     /**
      * The parent store.
@@ -193,7 +193,7 @@ public abstract class Folder {
     public URLName getURLName() throws MessagingException {
 	URLName storeURL = getStore().getURLName();
 	String fullname = getFullName();
-	StringBuffer encodedName = new StringBuffer();
+	StringBuilder encodedName = new StringBuilder();
 
 	if (fullname != null) {
 	    /*
@@ -646,6 +646,29 @@ public abstract class Folder {
      */
     public abstract void close(boolean expunge) throws MessagingException;
 
+    /**
+     * Close this Folder and expunge deleted messages. <p>
+     *
+     * A CLOSED ConnectionEvent is delivered to any ConnectionListeners
+     * registered on this Folder. Note that the folder is closed even
+     * if this method terminates abnormally by throwing a
+     * MessagingException. <p>
+     *
+     * This method supports the {@link java.lang.AutoCloseable AutoCloseable}
+     * interface. <p>
+     *
+     * This implementation calls <code>close(true)</code>.
+     *
+     * @exception	IllegalStateException if this folder is not opened
+     * @exception       MessagingException for other failures
+     * @see 		javax.mail.event.ConnectionEvent
+     * @since		JavaMail 1.6
+     */
+    @Override
+    public void close() throws MessagingException {
+	close(true);
+    }
+
     /**
      * Indicates whether this Folder is in the 'open' state.
      * @return  true if this Folder is in the 'open' state.
@@ -1288,7 +1311,7 @@ public abstract class Folder {
      */
     public Message[] search(SearchTerm term, Message[] msgs)
 				throws MessagingException {
-	List<Message> matchedMsgs = new ArrayList<Message>();
+	List<Message> matchedMsgs = new ArrayList<>();
 
 	// Run thru the given messages
 	for (Message msg : msgs) {
@@ -1337,7 +1360,7 @@ public abstract class Folder {
     public synchronized void
     addConnectionListener(ConnectionListener l) { 
    	if (connectionListeners == null) 
-	    connectionListeners = new Vector<ConnectionListener>();
+	    connectionListeners = new Vector<>();
 	connectionListeners.addElement(l);
     }
 
@@ -1403,7 +1426,7 @@ public abstract class Folder {
      */
     public synchronized void addFolderListener(FolderListener l) { 
    	if (folderListeners == null)
-	    folderListeners = new Vector<FolderListener>();
+	    folderListeners = new Vector<>();
 	folderListeners.addElement(l);
     }
 
@@ -1484,7 +1507,7 @@ public abstract class Folder {
      */
     public synchronized void addMessageCountListener(MessageCountListener l) { 
    	if (messageCountListeners == null)
-	    messageCountListeners = new Vector<MessageCountListener>();
+	    messageCountListeners = new Vector<>();
 	messageCountListeners.addElement(l);
     }
 
@@ -1574,7 +1597,7 @@ public abstract class Folder {
     public synchronized void
 			addMessageChangedListener(MessageChangedListener l) { 
    	if (messageChangedListeners == null)
-	    messageChangedListeners = new Vector<MessageChangedListener>();
+	    messageChangedListeners = new Vector<>();
 	messageChangedListeners.addElement(l);
     }
 
@@ -1632,6 +1655,7 @@ public abstract class Folder {
 	q.enqueue(event, v);
     }
 
+    @Override
     protected void finalize() throws Throwable {
 	try {
 	    q.terminateQueue();
@@ -1646,6 +1670,7 @@ public abstract class Folder {
      * the default toString() behavior.
      */
 
+    @Override
     public String toString() {
 	String s = getFullName();
 	if (s != null)
