@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2018 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -46,7 +46,12 @@ import java.lang.*;
 import java.io.*;
 import java.util.*;
 import java.text.ParseException;
-import com.sun.mail.util.*;
+import com.sun.mail.util.PropUtil;
+import com.sun.mail.util.ASCIIUtility;
+import com.sun.mail.util.MimeUtil;
+import com.sun.mail.util.MessageRemovedIOException;
+import com.sun.mail.util.FolderClosedIOException;
+import com.sun.mail.util.LineOutputStream;
 import javax.mail.util.SharedByteArrayInputStream;
 
 /**
@@ -178,6 +183,8 @@ public class MimeMessage extends Message implements MimePart {
 
     // Should addresses in headers be parsed in "strict" mode?
     private boolean strict = true;
+    // Is UTF-8 allowed in headers?
+    private boolean allowutf8 = false;
 
     /**
      * Default constructor. An empty message object is created.
@@ -316,9 +323,13 @@ public class MimeMessage extends Message implements MimePart {
      * Set the strict flag based on property.
      */
     private void initStrict() {
-	if (session != null)
-	    strict = PropUtil.getBooleanSessionProperty(session,
+	if (session != null) {
+	    Properties props = session.getProperties();
+	    strict = PropUtil.getBooleanProperty(props,
 				    "mail.mime.address.strict", true);
+	    allowutf8 = PropUtil.getBooleanProperty(props,
+				    "mail.mime.allowutf8", false);
+	}
     }
 
     /**
@@ -368,6 +379,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException for failures
      * @see	#headers
      */
+    @Override
     public Address[] getFrom() throws MessagingException {
 	Address[] a = getAddressHeader("From");
 	if (a == null)
@@ -389,6 +401,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setFrom(Address address) throws MessagingException {
 	if (address == null)
 	    removeHeader("From");
@@ -428,6 +441,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setFrom() throws MessagingException {
 	InternetAddress me = null;
 	try {
@@ -455,6 +469,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void addFrom(Address[] addresses) throws MessagingException {
 	addAddressHeader("From", addresses);
     }
@@ -520,6 +535,7 @@ public class MimeMessage extends Message implements MimePart {
 	    super(type);
 	}
 
+	@Override
 	protected Object readResolve() throws ObjectStreamException {
 	    if (type.equals("Newsgroups"))
 		return NEWSGROUPS;
@@ -556,6 +572,7 @@ public class MimeMessage extends Message implements MimePart {
      * @see		javax.mail.Message.RecipientType#BCC
      * @see		javax.mail.internet.MimeMessage.RecipientType#NEWSGROUPS
      */
+    @Override
     public Address[] getRecipients(Message.RecipientType type)
 				throws MessagingException {
 	if (type == RecipientType.NEWSGROUPS) {
@@ -576,6 +593,7 @@ public class MimeMessage extends Message implements MimePart {
      * @see		javax.mail.Message.RecipientType#BCC
      * @see		javax.mail.internet.MimeMessage.RecipientType#NEWSGROUPS
      */
+    @Override
     public Address[] getAllRecipients() throws MessagingException {
 	Address[] all = super.getAllRecipients();
 	Address[] ng = getRecipients(RecipientType.NEWSGROUPS);
@@ -606,6 +624,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException for other failures
      * @see		#getRecipients
      */
+    @Override
     public void setRecipients(Message.RecipientType type, Address[] addresses)
                                 throws MessagingException {
 	if (type == RecipientType.NEWSGROUPS) {
@@ -659,6 +678,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void addRecipients(Message.RecipientType type, Address[] addresses)
                                 throws MessagingException {
 	if (type == RecipientType.NEWSGROUPS) {
@@ -705,6 +725,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException for failures
      * @see		#headers
      */
+    @Override
     public Address[] getReplyTo() throws MessagingException {
 	Address[] a = getAddressHeader("Reply-To");
 	if (a == null || a.length == 0)
@@ -723,6 +744,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setReplyTo(Address[] addresses) throws MessagingException {
 	setAddressHeader("Reply-To", addresses);
     }
@@ -737,7 +759,11 @@ public class MimeMessage extends Message implements MimePart {
     // Convenience method to set addresses
     private void setAddressHeader(String name, Address[] addresses)
 			throws MessagingException {
-	String s = InternetAddress.toString(addresses, name.length() + 2);
+	String s;
+	if (allowutf8)
+	    s = InternetAddress.toUnicodeString(addresses, name.length() + 2);
+	else
+	    s = InternetAddress.toString(addresses, name.length() + 2);
 	if (s == null)
 	    removeHeader(name);
 	else
@@ -757,7 +783,11 @@ public class MimeMessage extends Message implements MimePart {
 	    System.arraycopy(a, 0, anew, 0, a.length);
 	    System.arraycopy(addresses, 0, anew, a.length, addresses.length);
 	}
-	String s = InternetAddress.toString(anew, name.length() + 2);
+	String s;
+	if (allowutf8)
+	    s = InternetAddress.toUnicodeString(anew, name.length() + 2);
+	else
+	    s = InternetAddress.toString(anew, name.length() + 2);
 	if (s == null)
 	    return;
 	setHeader(name, s);
@@ -778,6 +808,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException for failures
      * @see		#headers
      */
+    @Override
     public String getSubject() throws MessagingException {
 	String rawvalue = getHeader("Subject", null);
 
@@ -815,6 +846,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setSubject(String subject) throws MessagingException {
 	setSubject(subject, null);
     }
@@ -868,6 +900,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          The sent Date
      * @exception	MessagingException for failures
      */
+    @Override
     public Date getSentDate() throws MessagingException {
 	String s = getHeader("Date", null);
 	if (s != null) {
@@ -895,6 +928,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setSentDate(Date d) throws MessagingException {
 	if (d == null)
 	    removeHeader("Date");
@@ -918,6 +952,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          the date this message was received
      * @exception	MessagingException for failures
      */
+    @Override
     public Date getReceivedDate() throws MessagingException {
 	return null;	
     }
@@ -939,6 +974,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          size of content in bytes
      * @exception	MessagingException for failures
      */  
+    @Override
     public int getSize() throws MessagingException {
 	if (content != null)
 	    return content.length;
@@ -969,9 +1005,10 @@ public class MimeMessage extends Message implements MimePart {
      * @return          number of lines in the content.
      * @exception	MessagingException for failures
      */  
-     public int getLineCount() throws MessagingException {
+    @Override
+    public int getLineCount() throws MessagingException {
 	return -1;
-     }
+    }
 
     /**
      * Returns the value of the RFC 822 "Content-Type" header field. 
@@ -986,6 +1023,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException for failures
      * @see             javax.activation.DataHandler
      */
+    @Override
     public String getContentType() throws MessagingException {
 	String s = getHeader("Content-Type", null);
 	s = MimeUtil.cleanContentType(this, s);
@@ -1012,6 +1050,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return			true if it matches the MIME type
      * @exception		MessagingException for failures
      */
+    @Override
     public boolean isMimeType(String mimeType) throws MessagingException {
 	return MimeBodyPart.isMimeType(this, mimeType);
     }
@@ -1030,6 +1069,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          disposition of this part, or null if unknown
      * @exception	MessagingException for failures
      */
+    @Override
     public String getDisposition() throws MessagingException {
 	return MimeBodyPart.getDisposition(this);
     }
@@ -1045,6 +1085,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setDisposition(String disposition) throws MessagingException {
 	MimeBodyPart.setDisposition(this, disposition);
     }
@@ -1061,6 +1102,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          content-transfer-encoding
      * @exception	MessagingException for failures
      */
+    @Override
     public String getEncoding() throws MessagingException {
 	return MimeBodyPart.getEncoding(this);
     }
@@ -1076,6 +1118,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          content-ID
      * @exception	MessagingException for failures
      */
+    @Override
     public String getContentID() throws MessagingException {
 	return getHeader("Content-Id", null);
     }
@@ -1110,6 +1153,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return          content-MD5
      * @exception	MessagingException for failures
      */
+    @Override
     public String getContentMD5() throws MessagingException {
 	return getHeader("Content-MD5", null);
     }
@@ -1123,6 +1167,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception	MessagingException for other failures
      */
+    @Override
     public void setContentMD5(String md5) throws MessagingException {
 	setHeader("Content-MD5", md5);
     }
@@ -1143,6 +1188,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return	content-description
      * @exception	MessagingException for failures
      */
+    @Override
     public String getDescription() throws MessagingException {
 	return MimeBodyPart.getDescription(this);
     }
@@ -1172,6 +1218,7 @@ public class MimeMessage extends Message implements MimePart {
      *			in the exception chain if the charset
      *			conversion fails.
      */
+    @Override
     public void setDescription(String description) throws MessagingException {
 	setDescription(description, null);
     }
@@ -1219,6 +1266,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return			value of content-language header.
      * @exception		MessagingException for failures
      */
+    @Override
     public String[] getContentLanguage() throws MessagingException {
 	return MimeBodyPart.getContentLanguage(this);
     }
@@ -1234,6 +1282,7 @@ public class MimeMessage extends Message implements MimePart {
      *				obtained from a READ_ONLY folder.
      * @exception		MessagingException for other failures
      */
+    @Override
     public void setContentLanguage(String[] languages)
 			throws MessagingException {
 	MimeBodyPart.setContentLanguage(this, languages);
@@ -1277,6 +1326,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return	filename
      * @exception		MessagingException for failures
      */
+    @Override
     public String getFileName() throws MessagingException {
 	return MimeBodyPart.getFileName(this);
     }
@@ -1301,6 +1351,7 @@ public class MimeMessage extends Message implements MimePart {
      *				obtained from a READ_ONLY folder.
      * @exception		MessagingException for other failures
      */
+    @Override
     public void setFileName(String filename) throws MessagingException {
 	MimeBodyPart.setFileName(this, filename);	
     }
@@ -1338,6 +1389,7 @@ public class MimeMessage extends Message implements MimePart {
      * @see	#getContentStream
      * @see 	javax.activation.DataHandler#getInputStream
      */
+    @Override
     public InputStream getInputStream() 
 		throws IOException, MessagingException {
 	return getDataHandler().getInputStream();
@@ -1416,6 +1468,7 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception	MessagingException for failures
      */
+    @Override
     public synchronized DataHandler getDataHandler() 
 		throws MessagingException {
 	if (dh == null)
@@ -1446,6 +1499,7 @@ public class MimeMessage extends Message implements MimePart {
      *			javax.activation.DataHandler for more details.
      * @exception       MessagingException for other failures
      */
+    @Override
     public Object getContent() throws IOException, MessagingException {
 	if (cachedContent != null)
 	    return cachedContent;
@@ -1482,6 +1536,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public synchronized void setDataHandler(DataHandler dh) 
 		throws MessagingException {
 	this.dh = dh;
@@ -1508,6 +1563,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public void setContent(Object o, String type) 
 			throws MessagingException {
 	if (o instanceof Multipart)
@@ -1535,6 +1591,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException	if an error occurs
      * @see	#setText(String text, String charset)
      */
+    @Override
     public void setText(String text) throws MessagingException {
 	setText(text, null);
     }
@@ -1550,6 +1607,7 @@ public class MimeMessage extends Message implements MimePart {
      * @param	charset	the charset to use for the text
      * @exception	MessagingException	if an error occurs
      */
+    @Override
     public void setText(String text, String charset)
 			throws MessagingException {
 	MimeBodyPart.setText(this, text, charset, "plain");
@@ -1568,6 +1626,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception	MessagingException	if an error occurs
      * @since	JavaMail 1.4
      */
+    @Override
     public void setText(String text, String charset, String subtype)
                         throws MessagingException {
 	MimeBodyPart.setText(this, text, charset, subtype);
@@ -1584,6 +1643,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public void setContent(Multipart mp) throws MessagingException {
 	setDataHandler(new DataHandler(mp, mp.getContentType()));
 	mp.setParent(this);
@@ -1618,6 +1678,7 @@ public class MimeMessage extends Message implements MimePart {
      * @return		the reply Message
      * @exception	MessagingException for failures
      */
+    @Override
     public Message reply(boolean replyToAll) throws MessagingException {
 	return reply(replyToAll, true);
     }
@@ -1675,7 +1736,7 @@ public class MimeMessage extends Message implements MimePart {
 	Address a[] = getReplyTo();
 	reply.setRecipients(Message.RecipientType.TO, a);
 	if (replyToAll) {
-	    List<Address> v = new ArrayList<Address>();
+	    List<Address> v = new ArrayList<>();
 	    // add my own address to list
 	    InternetAddress me = InternetAddress.getLocalAddress(session);
 	    if (me != null)
@@ -1691,7 +1752,8 @@ public class MimeMessage extends Message implements MimePart {
 	    String replyallccStr = null;
 	    boolean replyallcc = false;
 	    if (session != null)
-		replyallcc = PropUtil.getBooleanSessionProperty(session,
+		replyallcc = PropUtil.getBooleanProperty(
+						session.getProperties(),
 						"mail.replyallcc", false);
 	    // add the recipients from the To field so far
 	    eliminateDuplicates(v, a);
@@ -1819,6 +1881,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception MessagingException for other failures
      * @see javax.activation.DataHandler#writeTo
      */
+    @Override
     public void writeTo(OutputStream os)
 				throws IOException, MessagingException {
 	writeTo(os, null);
@@ -1853,9 +1916,8 @@ public class MimeMessage extends Message implements MimePart {
 
 	// Else, the content is untouched, so we can just output it
 	// First, write out the header
-	@SuppressWarnings("unchecked")
 	Enumeration<String> hdrLines = getNonMatchingHeaderLines(ignoreList);
-	LineOutputStream los = new LineOutputStream(os);
+	LineOutputStream los = new LineOutputStream(os, allowutf8);
 	while (hdrLines.hasMoreElements())
 	    los.writeln(hdrLines.nextElement());
 
@@ -1898,6 +1960,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception       MessagingException for failures
      * @see 	javax.mail.internet.MimeUtility
      */
+    @Override
     public String[] getHeader(String name)
 			throws MessagingException {
 	return headers.getHeader(name);
@@ -1915,6 +1978,7 @@ public class MimeMessage extends Message implements MimePart {
      *				this name
      * @exception       	MessagingException for failures
      */
+    @Override
     public String getHeader(String name, String delimiter)
 				throws MessagingException {
 	return headers.getHeader(name, delimiter);
@@ -1936,6 +2000,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public void setHeader(String name, String value)
                                 throws MessagingException {
 	headers.setHeader(name, value);	
@@ -1956,6 +2021,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public void addHeader(String name, String value)
                                 throws MessagingException {
 	headers.addHeader(name, value);
@@ -1969,6 +2035,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception       MessagingException for other failures
      */
+    @Override
     public void removeHeader(String name)
                                 throws MessagingException {
 	headers.removeHeader(name);
@@ -1989,8 +2056,8 @@ public class MimeMessage extends Message implements MimePart {
      * @exception  MessagingException for failures
      * @see 	javax.mail.internet.MimeUtility
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getAllHeaders() throws MessagingException {
+    @Override
+    public Enumeration<Header> getAllHeaders() throws MessagingException {
 	return headers.getAllHeaders();	
     }
 
@@ -2001,8 +2068,8 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception  MessagingException for failures
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getMatchingHeaders(String[] names)
+    @Override
+    public Enumeration<Header> getMatchingHeaders(String[] names)
 			throws MessagingException {
 	return headers.getMatchingHeaders(names);
     }
@@ -2014,8 +2081,8 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception  MessagingException for failures
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getNonMatchingHeaders(String[] names)
+    @Override
+    public Enumeration<Header> getNonMatchingHeaders(String[] names)
 			throws MessagingException {
 	return headers.getNonMatchingHeaders(names);
     }
@@ -2029,6 +2096,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception  	MessagingException for other failures
      */
+    @Override
     public void addHeaderLine(String line) throws MessagingException {
 	headers.addHeaderLine(line);
     }
@@ -2040,8 +2108,8 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception  	MessagingException for failures
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getAllHeaderLines() throws MessagingException {
+    @Override
+    public Enumeration<String> getAllHeaderLines() throws MessagingException {
 	return headers.getAllHeaderLines();
     }
 
@@ -2052,8 +2120,8 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception  	MessagingException for failures
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getMatchingHeaderLines(String[] names)
+    @Override
+    public Enumeration<String> getMatchingHeaderLines(String[] names)
                                         throws MessagingException {
 	return headers.getMatchingHeaderLines(names);
     }
@@ -2065,8 +2133,8 @@ public class MimeMessage extends Message implements MimePart {
      *
      * @exception  	MessagingException for failures
      */
-    @SuppressWarnings("rawtypes")
-    public Enumeration getNonMatchingHeaderLines(String[] names)
+    @Override
+    public Enumeration<String> getNonMatchingHeaderLines(String[] names)
                                         throws MessagingException {
 	return headers.getNonMatchingHeaderLines(names);
     }
@@ -2083,6 +2151,7 @@ public class MimeMessage extends Message implements MimePart {
      * @exception  	MessagingException for failures
      * @see 		javax.mail.Flags
      */
+    @Override
     public synchronized Flags getFlags() throws MessagingException {
 	return (Flags)flags.clone();
     }
@@ -2105,6 +2174,7 @@ public class MimeMessage extends Message implements MimePart {
      * @see		javax.mail.Flags.Flag#RECENT
      * @see		javax.mail.Flags.Flag#SEEN
      */
+    @Override
     public synchronized boolean isSet(Flags.Flag flag)
 				throws MessagingException {
 	return (flags.contains(flag));
@@ -2121,6 +2191,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception  	MessagingException for other failures
      */
+    @Override
     public synchronized void setFlags(Flags flag, boolean set)
 			throws MessagingException {
 	if (set)
@@ -2153,6 +2224,7 @@ public class MimeMessage extends Message implements MimePart {
      *			obtained from a READ_ONLY folder.
      * @exception  	MessagingException for other failures
      */
+    @Override
     public void saveChanges() throws MessagingException {
 	modified = true;
 	saved = true;
@@ -2177,7 +2249,8 @@ public class MimeMessage extends Message implements MimePart {
      * Called by the <code>saveChanges</code> method to actually
      * update the MIME headers.  The implementation here sets the
      * <code>Content-Transfer-Encoding</code> header (if needed
-     * and not already set), the <code>MIME-Version</code> header
+     * and not already set), the <code>Date</code> header (if
+     * not already set), the <code>MIME-Version</code> header
      * and the <code>Message-ID</code> header. Also, if the content
      * of this message is a <code>MimeMultipart</code>, its
      * <code>updateHeaders</code> method is called. <p>
@@ -2197,6 +2270,8 @@ public class MimeMessage extends Message implements MimePart {
     protected synchronized void updateHeaders() throws MessagingException {
 	MimeBodyPart.updateHeaders(this);	
 	setHeader("MIME-Version", "1.0");
+	if (getHeader("Date") == null)
+	    setSentDate(new Date());
         updateMessageID();
 
 	if (cachedContent != null) {
@@ -2226,7 +2301,7 @@ public class MimeMessage extends Message implements MimePart {
      */
     protected InternetHeaders createInternetHeaders(InputStream is)
 				throws MessagingException {
-	return new InternetHeaders(is);
+	return new InternetHeaders(is, allowutf8);
     }
 
     /**
