@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2013 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -263,7 +263,8 @@ public class MimeMultipart extends Multipart {
      * boundary and creates MimeBodyParts for each part of the stream.
      *
      * @param	ds	DataSource, can be a MultipartDataSource
-     * @exception	MessagingException for failures
+     * @exception	ParseException for failures parsing the message
+     * @exception	MessagingException for other failures
      */
     public MimeMultipart(DataSource ds) throws MessagingException {
 	super();
@@ -330,6 +331,7 @@ public class MimeMultipart extends Multipart {
      *
      * @return		number of parts
      */
+    @Override
     public synchronized int getCount() throws MessagingException {
 	parse();
 	return super.getCount();
@@ -342,6 +344,7 @@ public class MimeMultipart extends Multipart {
      * @return		the Part
      * @exception       MessagingException if no such BodyPart exists
      */
+    @Override
     public synchronized BodyPart getBodyPart(int index) 
 			throws MessagingException {
 	parse();
@@ -381,6 +384,7 @@ public class MimeMultipart extends Multipart {
      *			implementation does not support modification
      *			of existing values
      */
+    @Override
     public boolean removeBodyPart(BodyPart part) throws MessagingException {
 	parse();
 	return super.removeBodyPart(part);
@@ -398,6 +402,7 @@ public class MimeMultipart extends Multipart {
      *			of existing values
      * @exception	MessagingException for other failures
      */
+    @Override
     public void removeBodyPart(int index) throws MessagingException {
 	parse();
 	super.removeBodyPart(index);
@@ -413,6 +418,7 @@ public class MimeMultipart extends Multipart {
      *			of existing values
      * @exception       MessagingException for other failures
      */
+    @Override
     public synchronized void addBodyPart(BodyPart part) 
 		throws MessagingException {
 	parse();
@@ -433,6 +439,7 @@ public class MimeMultipart extends Multipart {
      *			of existing values
      * @exception       MessagingException for other failures
      */
+    @Override
     public synchronized void addBodyPart(BodyPart part, int index) 
 				throws MessagingException {
 	parse();
@@ -519,6 +526,7 @@ public class MimeMultipart extends Multipart {
      * Iterates through all the parts and outputs each MIME part
      * separated by a boundary.
      */
+    @Override
     public synchronized void writeTo(OutputStream os)
 				throws IOException, MessagingException {
 	parse();
@@ -568,7 +576,8 @@ public class MimeMultipart extends Multipart {
      * The {@link #initializeProperties} method is called before
      * parsing the data.
      *
-     * @exception	MessagingException for failures
+     * @exception	ParseException for failures parsing the message
+     * @exception	MessagingException for other failures
      * @since	JavaMail 1.2
      */
     protected synchronized void parse() throws MessagingException {
@@ -602,12 +611,12 @@ public class MimeMultipart extends Multipart {
 	}
 	if (boundary == null && !ignoreMissingBoundaryParameter &&
 		!ignoreExistingBoundaryParameter)
-	    throw new MessagingException("Missing boundary parameter");
+	    throw new ParseException("Missing boundary parameter");
 
 	try {
 	    // Skip and save the preamble
 	    LineInputStream lin = new LineInputStream(in);
-	    StringBuffer preamblesb = null;
+	    StringBuilder preamblesb = null;
 	    String line;
 	    String lineSeparator = null;
 	    while ((line = lin.readLine()) != null) {
@@ -667,7 +676,7 @@ public class MimeMultipart extends Multipart {
 		    }
 		    // accumulate the preamble
 		    if (preamblesb == null)
-			preamblesb = new StringBuffer(line.length() + 2);
+			preamblesb = new StringBuilder(line.length() + 2);
 		    preamblesb.append(line).append(lineSeparator);
 		}
 	    }
@@ -679,7 +688,7 @@ public class MimeMultipart extends Multipart {
 		if (allowEmpty)
 		    return;
 		else
-		    throw new MessagingException("Missing start boundary");
+		    throw new ParseException("Missing start boundary");
 	    }
 
 	    // save individual boundary bytes for comparison later
@@ -731,7 +740,7 @@ public class MimeMultipart extends Multipart {
 			;
 		    if (line == null) {
 			if (!ignoreMissingEndBoundary)
-			    throw new MessagingException(
+			    throw new ParseException(
 					"missing multipart end boundary");
 			// assume there's just a missing end boundary
 			complete = false;
@@ -779,7 +788,7 @@ public class MimeMultipart extends Multipart {
 		    if (inSize < bl) {
 			// hit EOF
 			if (!ignoreMissingEndBoundary)
-			    throw new MessagingException(
+			    throw new ParseException(
 					"missing multipart end boundary");
 			if (sin != null)
 			    end = sin.getPosition();
