@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2009-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -45,7 +45,7 @@ import java.util.ArrayList;
 import java.io.IOException;
 import java.net.ServerSocket;
 import java.net.Socket;
-//import javax.net.ssl.SSLServerSocketFactory;
+import java.net.InetSocketAddress;
 import javax.net.ssl.*;
 
 /**
@@ -67,6 +67,8 @@ public final class TestServer extends Thread {
     /** Protocol handler. */
     private final ProtocolHandler handler;
 
+    private List<Thread> clients = new ArrayList<Thread>();
+
     /**
      * Test server.
      *
@@ -85,23 +87,54 @@ public final class TestServer extends Thread {
     public TestServer(final ProtocolHandler handler, final boolean isSSL)
 				throws IOException {
         this.handler = handler;
+
+	/*
+	 * Allowing the JDK to pick a port number sometimes results in it
+	 * picking a number that's already in use by another process, but
+	 * no error is returned.  Picking it ourself allows us to make sure
+	 * that it's not used before we pick it.  Hopefully the socket
+	 * creation will fail if the port is already in use.
+	 *
+	 * XXX - perhaps we should use Random to choose a port number in
+	 * the emphemeral range, in case a lot of low port numbers are
+	 * already in use.
+	 */
+	for (int port = 49152; port < 50000 /*65535*/; port++) {
+	    /*
+	    if (isListening(port))
+		continue;
+	    */
+	    try {
+		serverSocket = createServerSocket(port, isSSL);
+		return;
+	    } catch (IOException ex) {
+		// ignore
+	    }
+	}
+	throw new RuntimeException("Can't find unused port");
+    }
+
+    private static ServerSocket createServerSocket(int port, boolean isSSL)
+				throws IOException {
+	ServerSocket ss;
 	if (isSSL) {
 	    SSLServerSocketFactory sf =
 		(SSLServerSocketFactory)SSLServerSocketFactory.getDefault();
-	    serverSocket = sf.createServerSocket(0);
+	    ss = sf.createServerSocket(port);
 	    // enable only the anonymous cipher suites so we don't have to
 	    // create a server certificate
-	    List<String> anon = new ArrayList<String>();
+	    List<String> anon = new ArrayList<>();
 	    String[] suites = sf.getSupportedCipherSuites();
 	    for (int i = 0; i < suites.length; i++) {
 		if (suites[i].indexOf("_anon_") >= 0) {
 		    anon.add(suites[i]);
 		}
 	    }
-	    ((SSLServerSocket)serverSocket).setEnabledCipherSuites(
-					anon.toArray(new String[anon.size()]));
+	    ((SSLServerSocket)ss).setEnabledCipherSuites(
+				    anon.toArray(new String[anon.size()]));
 	} else
-	    serverSocket = new ServerSocket(0);
+	    ss = new ServerSocket(port);
+	return ss;
     }
 
     /**
@@ -129,6 +162,26 @@ public final class TestServer extends Thread {
     /**
      * {@inheritDoc}
      */
+    @Override
+    public void start() {
+	super.start();
+	// don't return until server is really listening
+	// XXX - this might not be necessary
+	for (int tries = 0; tries < 10; tries++) {
+	    if (isListening(getPort())) {
+		return;
+	    }
+	    try {
+		Thread.sleep(100);
+	    } catch (InterruptedException ex) { }
+	}
+	throw new RuntimeException("Server isn't listening");
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
     public void run() {
         try {
             keepOn = true;
@@ -139,7 +192,11 @@ public final class TestServer extends Thread {
                     final ProtocolHandler pHandler =
 			(ProtocolHandler)handler.clone();
                     pHandler.setClientSocket(clientSocket);
-                    new Thread(pHandler).start();
+                    Thread t = new Thread(pHandler);
+		    synchronized (clients) {
+			clients.add(t);
+		    }
+		    t.start();
                 } catch (final IOException e) {
                     //e.printStackTrace();
                 }
@@ -148,4 +205,49 @@ public final class TestServer extends Thread {
             quit();
         }
     }
+
+    /**
+     * Return number of clients ever created.
+     */
+    public int clientCount() {
+	synchronized (clients) {
+	    // isListening creates a client that we don't count
+	    return clients.size() - 1;
+	}
+    }
+
+    /**
+     * Wait for at least n clients to terminate.
+     */
+    public void waitForClients(int n) {
+	if (n > clientCount())
+	    throw new RuntimeException("not that many clients");
+	for (;;) {
+	    int num = -1;	// ignore isListening client
+	    synchronized (clients) {
+		for (Thread t : clients) {
+		    if (!t.isAlive()) {
+			if (++num >= n)
+			    return;
+		    }
+		}
+	    }
+	    try {
+		Thread.sleep(100);
+	    } catch (InterruptedException ex) { }
+	}
+    }
+
+    private boolean isListening(int port) {
+	try {
+	    Socket s = new Socket();
+	    s.connect(new InetSocketAddress("localhost", port), 100);
+	    // it's listening!
+	    s.close();
+	    return true;
+	} catch (Exception ex) {
+	    //System.out.println(ex);
+	}
+	return false;
+    }
 }
