@@ -1,20 +1,20 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2016 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2016 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2017 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2017 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -634,9 +634,9 @@ public class CollectorFormatterTest extends AbstractLogging {
 
     @Test
     public void testGetTailExample3a() {
-        String p = "These {3} messages occurred between "
-                + "{9,choice,86400000#{7,date} {7,time} and {8,time}"
-                + "|86400000<{7,date} and {8,date}}\n";
+        String p = "These {3} messages occurred between\n"
+                + "{7,date,EEE, MMM dd HH:mm:ss:S ZZZ yyyy}"
+                + " and {8,time,EEE, MMM dd HH:mm:ss:S ZZZ yyyy}\n";
         CollectorFormatter cf = new CollectorFormatter(p);
         LogRecord min = new LogRecord(Level.SEVERE, "");
         setEpochMilli(min, 1248203502449L);
@@ -654,21 +654,9 @@ public class CollectorFormatterTest extends AbstractLogging {
 
         String output = cf.getTail((Handler) null);
         assertNotNull(output);
-
-        cf.format(min);
-        for (int i = 0; i < 114; ++i) {
-            LogRecord mid = new LogRecord(Level.SEVERE, "");
-            setEpochMilli(mid, min.getMillis());
-            cf.format(mid);
-        }
-
-        setEpochMilli(max, min.getMillis() + 2591000000L);
-        cf.format(max);
-
-        output = cf.getTail((Handler) null);
-        assertNotNull(output);
     }
 
+    @Test
     public void testGetTailExample3b() {
         String p = "These {3} messages occurred between "
                 + "{9,choice,86400000#{7,date} {7,time} and {8,time}"
