@@ -1,19 +1,19 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
+ * https://oss.oracle.com/licenses/CDDL+GPL-1.1
+ * or LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
  * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
+ * file and include the License file at LICENSE.txt.
  *
  * GPL Classpath Exception:
  * Oracle designates this particular file as subject to the "Classpath"
@@ -82,7 +82,7 @@ public class HeaderTokenizerTest {
     @Parameters
     public static Collection<Object[]> data() throws IOException {
 	junit = true;
-	testData = new ArrayList<Object[]>();
+	testData = new ArrayList<>();
 	parse(new BufferedReader(new InputStreamReader(
 	    InternetAddressTest.class.getResourceAsStream("tokenlist"))));
 	return testData;
@@ -231,7 +231,7 @@ public class HeaderTokenizerTest {
 			!return_comments);
 	    HeaderTokenizer.Token tok;
 	    Vector<HeaderTokenizer.Token> toklist
-		    = new Vector<HeaderTokenizer.Token>();
+		    = new Vector<>();
 	    while ((tok = ht.next()).getType() != HeaderTokenizer.Token.EOF)
 		toklist.addElement(tok);
 	    if (gen_test_input)
